@@ -1,18 +1,17 @@
 /* This file is part of Patchage.
- * Copyright (C) 2008 Nedko Arnaudov <nedko@arnaudov.name>
+ * Copyright 2008 Nedko Arnaudov <nedko@arnaudov.name>
  *
- * Patchage is free software; you can redistribute it and/or modify it under the
- * terms of the GNU General Public License as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any later
- * version.
+ * Patchage is free software: you can redistribute it and/or modify it under
+ * the terms of the GNU General Public License as published by the Free
+ * Software Foundation, either version 3 of the License, or (at your option)
+ * any later version.
  *
  * Patchage is distributed in the hope that it will be useful, but WITHOUT ANY
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
+ * You should have received a copy of the GNU General Public License
+ * along with Patchage.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <cassert>
@@ -20,7 +19,7 @@
 #include <string>
 #include <set>
 
-#include "patchage-config.h"
+#include "patchage_config.h"
 
 #include <glib.h>
 #include <dbus/dbus.h>
@@ -29,9 +28,6 @@
 
 #include <boost/format.hpp>
 
-#include "raul/log.hpp"
-#include "raul/SharedPtr.hpp"
-
 #include "Driver.hpp"
 #include "JackDbusDriver.hpp"
 #include "Patchage.hpp"
@@ -39,39 +35,34 @@
 #include "PatchageEvent.hpp"
 #include "PatchageModule.hpp"
 
-#define JACKDBUS_SERVICE         "org.jackaudio.service"
-#define JACKDBUS_OBJECT          "/org/jackaudio/Controller"
-#define JACKDBUS_IFACE_CONTROL   "org.jackaudio.JackControl"
-#define JACKDBUS_IFACE_PATCHBAY  "org.jackaudio.JackPatchbay"
+#define JACKDBUS_SERVICE        "org.jackaudio.service"
+#define JACKDBUS_OBJECT         "/org/jackaudio/Controller"
+#define JACKDBUS_IFACE_CONTROL  "org.jackaudio.JackControl"
+#define JACKDBUS_IFACE_PATCHBAY "org.jackaudio.JackPatchbay"
+
 #define JACKDBUS_CALL_DEFAULT_TIMEOUT 1000 // in milliseconds
 
-#define JACKDBUS_PORT_FLAG_INPUT         0x00000001
-#define JACKDBUS_PORT_FLAG_OUTPUT        0x00000002
-#define JACKDBUS_PORT_FLAG_PHYSICAL      0x00000004
-#define JACKDBUS_PORT_FLAG_CAN_MONITOR   0x00000008
-#define JACKDBUS_PORT_FLAG_TERMINAL      0x00000010
+#define JACKDBUS_PORT_FLAG_INPUT       0x00000001
+#define JACKDBUS_PORT_FLAG_OUTPUT      0x00000002
+#define JACKDBUS_PORT_FLAG_PHYSICAL    0x00000004
+#define JACKDBUS_PORT_FLAG_CAN_MONITOR 0x00000008
+#define JACKDBUS_PORT_FLAG_TERMINAL    0x00000010
 
-#define JACKDBUS_PORT_TYPE_AUDIO    0
-#define JACKDBUS_PORT_TYPE_MIDI     1
-
-//#define LOG_TO_STD
-#define LOG_TO_STATUS
+#define JACKDBUS_PORT_TYPE_AUDIO 0
+#define JACKDBUS_PORT_TYPE_MIDI  1
 
 //#define USE_FULL_REFRESH
 
-
 JackDriver::JackDriver(Patchage* app)
 	: _app(app)
 	, _dbus_connection(0)
 	, _server_responding(false)
 	, _server_started(false)
 	, _graph_version(0)
-	, _max_dsp_load(0.0)
 {
 	dbus_error_init(&_dbus_error);
 }
 
-
 JackDriver::~JackDriver()
 {
 	if (_dbus_connection) {
@@ -83,35 +74,20 @@ JackDriver::~JackDriver()
 	}
 }
 
+static bool
+is_jack_port(const PatchagePort* port)
+{
+	return port->type() == JACK_AUDIO || port->type() == JACK_MIDI;
+}
 
 /** Destroy all JACK (canvas) ports.
  */
 void
 JackDriver::destroy_all()
 {
-	ItemList modules = _app->canvas()->items(); // copy
-	for (ItemList::iterator m = modules.begin(); m != modules.end(); ++m) {
-		SharedPtr<Module> module = PtrCast<Module>(*m);
-		if (!module)
-			continue;
-
-		PortVector ports = module->ports(); // copy
-		for (PortVector::iterator p = ports.begin(); p != ports.end(); ++p) {
-			SharedPtr<PatchagePort> port = boost::dynamic_pointer_cast<PatchagePort>(*p);
-			if (port && (port->type() == JACK_AUDIO || port->type() == JACK_MIDI)) {
-				module->remove_port(port);
-				port->hide();
-			}
-		}
-
-		if (module->ports().empty())
-			_app->canvas()->remove_item(module);
-		else
-			_app->enqueue_resize(module);
-	}
+	_app->canvas()->remove_ports(is_jack_port);
 }
 
-
 void
 JackDriver::update_attached()
 {
@@ -136,7 +112,6 @@ JackDriver::update_attached()
 	}
 }
 
-
 void
 JackDriver::on_jack_appeared()
 {
@@ -144,7 +119,6 @@ JackDriver::on_jack_appeared()
 	update_attached();
 }
 
-
 void
 JackDriver::on_jack_disappeared()
 {
@@ -161,13 +135,11 @@ JackDriver::on_jack_disappeared()
 	_server_started = false;
 }
 
-
 /** Handle signals we have subscribed for in attach(). */
 DBusHandlerResult
-JackDriver::dbus_message_hook(
-	DBusConnection* connection,
-	DBusMessage*    message,
-	void*           jack_driver)
+JackDriver::dbus_message_hook(DBusConnection* connection,
+                              DBusMessage*    message,
+                              void*           jack_driver)
 {
 	const char*   client2_name;
 	const char*   client_name;
@@ -367,7 +339,6 @@ JackDriver::dbus_message_hook(
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
-
 bool
 JackDriver::call(
 	bool          response_expected,
@@ -416,7 +387,6 @@ JackDriver::call(
 	return reply_ptr;
 }
 
-
 bool
 JackDriver::is_started()
 {
@@ -441,7 +411,6 @@ JackDriver::is_started()
 	return started;
 }
 
-
 void
 JackDriver::start_server()
 {
@@ -456,7 +425,6 @@ JackDriver::start_server()
 	update_attached();
 }
 
-
 void
 JackDriver::stop_server()
 {
@@ -474,7 +442,6 @@ JackDriver::stop_server()
 	}
 }
 
-
 void
 JackDriver::attach(bool launch_daemon)
 {
@@ -513,53 +480,43 @@ JackDriver::attach(bool launch_daemon)
 	}
 }
 
-
 void
 JackDriver::detach()
 {
 	stop_server();
 }
 
-
 bool
 JackDriver::is_attached() const
 {
 	return _dbus_connection && _server_responding;
 }
 
-
 void
-JackDriver::add_port(
-	SharedPtr<PatchageModule>& module,
-	PortType                   type,
-	const std::string&         name,
-	bool                       is_input)
+JackDriver::add_port(PatchageModule*    module,
+                     PortType           type,
+                     const std::string& name,
+                     bool               is_input)
 {
 	if (module->get_port(name)) {
 		return;
 	}
 
-	module->add_port(
-		SharedPtr<PatchagePort>(
-			new PatchagePort(
-				module,
-				type,
-				name,
-				is_input,
-				_app->state_manager()->get_port_color(type))));
-
-	_app->enqueue_resize(module);
+	new PatchagePort(
+		*module,
+		type,
+		name,
+		is_input,
+		_app->state_manager()->get_port_color(type));
 }
 
-
 void
-JackDriver::add_port(
-	dbus_uint64_t client_id,
-	const char*   client_name,
-	dbus_uint64_t port_id,
-	const char*   port_name,
-	dbus_uint32_t port_flags,
-	dbus_uint32_t port_type)
+JackDriver::add_port(dbus_uint64_t client_id,
+                     const char*   client_name,
+                     dbus_uint64_t port_id,
+                     const char*   port_name,
+                     dbus_uint32_t port_flags,
+                     dbus_uint32_t port_type)
 {
 	PortType local_port_type;
 
@@ -584,39 +541,33 @@ JackDriver::add_port(
 		}
 	}
 
-	SharedPtr<PatchageModule> module = find_or_create_module(type, client_name);
+	PatchageModule* module = find_or_create_module(type, client_name);
 
 	add_port(module, local_port_type, port_name, port_flags & JACKDBUS_PORT_FLAG_INPUT);
 }
 
-
 void
-JackDriver::remove_port(
-	dbus_uint64_t client_id,
-	const char*   client_name,
-	dbus_uint64_t port_id,
-	const char*   port_name)
+JackDriver::remove_port(dbus_uint64_t client_id,
+                        const char*   client_name,
+                        dbus_uint64_t port_id,
+                        const char*   port_name)
 {
-	SharedPtr<PatchagePort> port = _app->canvas()->find_port_by_name(client_name, port_name);
+	PatchagePort* port = _app->canvas()->find_port_by_name(client_name, port_name);
 	if (!port) {
 		error_msg("Unable to remove unknown port");
 		return;
 	}
 
-	SharedPtr<PatchageModule> module = PtrCast<PatchageModule>(port->module().lock());
+	PatchageModule* module = dynamic_cast<PatchageModule*>(port->get_module());
 
-	module->remove_port(port);
-	port.reset();
+	delete port;
 
 	// No empty modules (for now)
 	if (module->num_ports() == 0) {
-		_app->canvas()->remove_item(module);
-		module.reset();
-	} else {
-		_app->enqueue_resize(module);
+		delete module;
 	}
 
-	if (_app->canvas()->items().empty()) {
+	if (_app->canvas()->empty()) {
 		if (_server_started) {
 			signal_detached.emit();
 		}
@@ -625,16 +576,15 @@ JackDriver::remove_port(
 	}
 }
 
-
-SharedPtr<PatchageModule>
+PatchageModule*
 JackDriver::find_or_create_module(
 	ModuleType type,
 	const std::string& name)
 {
-	SharedPtr<PatchageModule> module = _app->canvas()->find_module(name, type);
+	PatchageModule* module = _app->canvas()->find_module(name, type);
 
 	if (!module) {
-		module = SharedPtr<PatchageModule>(new PatchageModule(_app, name, type));
+		module = new PatchageModule(_app, name, type);
 		module->load_location();
 		_app->canvas()->add_module(name, module);
 	}
@@ -642,63 +592,58 @@ JackDriver::find_or_create_module(
 	return module;
 }
 
-
 void
-JackDriver::connect_ports(
-	dbus_uint64_t connection_id,
-	dbus_uint64_t client1_id,
-	const char*   client1_name,
-	dbus_uint64_t port1_id,
-	const char*   port1_name,
-	dbus_uint64_t client2_id,
-	const char*   client2_name,
-	dbus_uint64_t port2_id,
-	const char*   port2_name)
+JackDriver::connect_ports(dbus_uint64_t connection_id,
+                          dbus_uint64_t client1_id,
+                          const char*   client1_name,
+                          dbus_uint64_t port1_id,
+                          const char*   port1_name,
+                          dbus_uint64_t client2_id,
+                          const char*   client2_name,
+                          dbus_uint64_t port2_id,
+                          const char*   port2_name)
 {
-	SharedPtr<PatchagePort> port1 = _app->canvas()->find_port_by_name(client1_name, port1_name);
+	PatchagePort* port1 = _app->canvas()->find_port_by_name(client1_name, port1_name);
 	if (!port1) {
-		error_msg((string)"Unable to connect unknown port '" + port1_name + "' of client '" + client1_name + "'");
+		error_msg((std::string)"Unable to connect unknown port '" + port1_name + "' of client '" + client1_name + "'");
 		return;
 	}
 
-	SharedPtr<PatchagePort> port2 = _app->canvas()->find_port_by_name(client2_name, port2_name);
+	PatchagePort* port2 = _app->canvas()->find_port_by_name(client2_name, port2_name);
 	if (!port2) {
-		error_msg((string)"Unable to connect unknown port '" + port2_name + "' of client '" + client2_name + "'");
+		error_msg((std::string)"Unable to connect unknown port '" + port2_name + "' of client '" + client2_name + "'");
 		return;
 	}
 
-	_app->canvas()->add_connection(port1, port2, port1->color() + 0x22222200);
+	_app->canvas()->connect(port1, port2);
 }
 
-
 void
-JackDriver::disconnect_ports(
-	dbus_uint64_t connection_id,
-	dbus_uint64_t client1_id,
-	const char*   client1_name,
-	dbus_uint64_t port1_id,
-	const char*   port1_name,
-	dbus_uint64_t client2_id,
-	const char*   client2_name,
-	dbus_uint64_t port2_id,
-	const char*   port2_name)
+JackDriver::disconnect_ports(dbus_uint64_t connection_id,
+                             dbus_uint64_t client1_id,
+                             const char*   client1_name,
+                             dbus_uint64_t port1_id,
+                             const char*   port1_name,
+                             dbus_uint64_t client2_id,
+                             const char*   client2_name,
+                             dbus_uint64_t port2_id,
+                             const char*   port2_name)
 {
-	SharedPtr<PatchagePort> port1 = _app->canvas()->find_port_by_name(client1_name, port1_name);
+	PatchagePort* port1 = _app->canvas()->find_port_by_name(client1_name, port1_name);
 	if (!port1) {
-		error_msg((string)"Unable to disconnect unknown port '" + port1_name + "' of client '" + client1_name + "'");
+		error_msg((std::string)"Unable to disconnect unknown port '" + port1_name + "' of client '" + client1_name + "'");
 		return;
 	}
 
-	SharedPtr<PatchagePort> port2 = _app->canvas()->find_port_by_name(client2_name, port2_name);
+	PatchagePort* port2 = _app->canvas()->find_port_by_name(client2_name, port2_name);
 	if (!port2) {
-		error_msg((string)"Unable to disconnect unknown port '" + port2_name + "' of client '" + client2_name + "'");
+		error_msg((std::string)"Unable to disconnect unknown port '" + port2_name + "' of client '" + client2_name + "'");
 		return;
 	}
 
-	_app->canvas()->remove_connection(port1, port2);
+	_app->canvas()->disconnect(port1, port2);
 }
 
-
 void
 JackDriver::refresh_internal(bool force)
 {
@@ -738,7 +683,7 @@ JackDriver::refresh_internal(bool force)
 	reply_signature = dbus_message_get_signature(reply_ptr);
 
 	if (strcmp(reply_signature, "ta(tsa(tsuu))a(tstststst)") != 0) {
-		error_msg((string )"GetGraph() reply signature mismatch. " + reply_signature);
+		error_msg((std::string)"GetGraph() reply signature mismatch. " + reply_signature);
 		goto unref;
 	}
 
@@ -857,30 +802,22 @@ unref:
 	dbus_message_unref(reply_ptr);
 }
 
-
 void
 JackDriver::refresh()
 {
 	refresh_internal(true);
 }
 
-
 bool
-JackDriver::connect(
-	SharedPtr<PatchagePort> src,
-	SharedPtr<PatchagePort> dst)
+JackDriver::connect(PatchagePort* src,
+                    PatchagePort* dst)
 {
-	const char*  client1_name;
-	const char*  port1_name;
-	const char*  client2_name;
-	const char*  port2_name;
+	const char* client1_name = src->get_module()->get_label();
+	const char* port1_name   = src->get_label();
+	const char* client2_name = dst->get_module()->get_label();
+	const char* port2_name   = dst->get_label();
+
 	DBusMessage* reply_ptr;
-
-	client1_name = src->module().lock()->name().c_str();
-	port1_name   = src->name().c_str();
-	client2_name = dst->module().lock()->name().c_str();
-	port2_name   = dst->name().c_str();
-
 	if (!call(true, JACKDBUS_IFACE_PATCHBAY, "ConnectPortsByName", &reply_ptr,
 				DBUS_TYPE_STRING, &client1_name,
 				DBUS_TYPE_STRING, &port1_name,
@@ -894,23 +831,16 @@ JackDriver::connect(
 	return true;
 }
 
-
 bool
-JackDriver::disconnect(
-	SharedPtr<PatchagePort> src,
-	SharedPtr<PatchagePort> dst)
+JackDriver::disconnect(PatchagePort* src,
+                       PatchagePort* dst)
 {
-	const char*  client1_name;
-	const char*  port1_name;
-	const char*  client2_name;
-	const char*  port2_name;
+	const char* client1_name = src->get_module()->get_label();
+	const char* port1_name   = src->get_label();
+	const char* client2_name = dst->get_module()->get_label();
+	const char* port2_name   = dst->get_label();
+
 	DBusMessage* reply_ptr;
-
-	client1_name = src->module().lock()->name().c_str();
-	port1_name   = src->name().c_str();
-	client2_name = dst->module().lock()->name().c_str();
-	port2_name   = dst->name().c_str();
-
 	if (!call(true, JACKDBUS_IFACE_PATCHBAY, "DisconnectPortsByName", &reply_ptr,
 				DBUS_TYPE_STRING, &client1_name,
 				DBUS_TYPE_STRING, &port1_name,
@@ -924,7 +854,6 @@ JackDriver::disconnect(
 	return true;
 }
 
-
 jack_nframes_t
 JackDriver::buffer_size()
 {
@@ -954,7 +883,6 @@ fail:
 	return 4096; // something fake, patchage needs it to match combobox value
 }
 
-
 bool
 JackDriver::set_buffer_size(jack_nframes_t size)
 {
@@ -972,7 +900,6 @@ JackDriver::set_buffer_size(jack_nframes_t size)
 	return true;
 }
 
-
 float
 JackDriver::sample_rate()
 {
@@ -995,20 +922,22 @@ JackDriver::sample_rate()
 	return sample_rate;
 }
 
-
 bool
 JackDriver::is_realtime() const
 {
 	DBusMessage* reply_ptr;
-	dbus_bool_t realtime;
+	dbus_bool_t  realtime;
 
-	if (!this->call(true, JACKDBUS_IFACE_CONTROL, "IsRealtime", &reply_ptr, DBUS_TYPE_INVALID)) {
+	JackDriver* me = const_cast<JackDriver*>(this);
+	if (!me->call(true, JACKDBUS_IFACE_CONTROL, "IsRealtime",
+	              &reply_ptr, DBUS_TYPE_INVALID)) {
 		return false;
 	}
 
-	if (!dbus_message_get_args(reply_ptr, &this->_dbus_error, DBUS_TYPE_BOOLEAN, &realtime, DBUS_TYPE_INVALID)) {
+	if (!dbus_message_get_args(reply_ptr, &me->_dbus_error, DBUS_TYPE_BOOLEAN,
+	                           &realtime, DBUS_TYPE_INVALID)) {
 		dbus_message_unref(reply_ptr);
-		dbus_error_free(&this->_dbus_error);
+		dbus_error_free(&me->_dbus_error);
 		error_msg("decoding reply of IsRealtime failed.");
 		return false;
 	}
@@ -1018,7 +947,6 @@ JackDriver::is_realtime() const
 	return realtime;
 }
 
-
 size_t
 JackDriver::get_xruns()
 {
@@ -1045,7 +973,6 @@ JackDriver::get_xruns()
 	return xruns;
 }
 
-
 void
 JackDriver::reset_xruns()
 {
@@ -1058,78 +985,23 @@ JackDriver::reset_xruns()
 	dbus_message_unref(reply_ptr);
 }
 
-
-float
-JackDriver::get_max_dsp_load()
-{
-	DBusMessage* reply_ptr;
-	double load;
-
-	if (_server_responding && !_server_started) {
-		return 0.0;
-	}
-
-	if (!call(true, JACKDBUS_IFACE_CONTROL, "GetLoad", &reply_ptr, DBUS_TYPE_INVALID)) {
-		return 0.0;
-	}
-
-	if (!dbus_message_get_args(reply_ptr, &_dbus_error, DBUS_TYPE_DOUBLE, &load, DBUS_TYPE_INVALID)) {
-		dbus_message_unref(reply_ptr);
-		dbus_error_free(&_dbus_error);
-		error_msg("decoding reply of GetLoad failed.");
-		return 0.0;
-	}
-
-	dbus_message_unref(reply_ptr);
-
-	load /= 100.0;								// dbus returns it in percents, we use 0..1
-
-	if (load > _max_dsp_load) {
-		_max_dsp_load = load;
-	}
-
-	return _max_dsp_load;
-}
-
-
-void
-JackDriver::reset_max_dsp_load()
-{
-	_max_dsp_load = 0.0;
-}
-
-
-SharedPtr<PatchagePort>
-JackDriver::create_port_view(
-	Patchage* patchage,
-	const PortID& id)
+PatchagePort*
+JackDriver::create_port_view(Patchage*     patchage,
+                             const PortID& id)
 {
 	assert(false);  // we dont use events at all
+	return NULL;
 }
 
-
 void
 JackDriver::error_msg(const std::string& msg) const
 {
-#if defined(LOG_TO_STATUS)
-	_app->status_msg((std::string)"[JACKDBUS] " + msg);
-#endif
-
-#if defined(LOG_TO_STD)
-	cerr << (std::string)"[JACKDBUS] " << msg << endl;
-#endif
+	_app->error_msg((std::string)"Jack: " + msg);
 }
 
-
 void
 JackDriver::info_msg(const std::string& msg) const
 {
-#if defined(LOG_TO_STATUS)
-	_app->status_msg((std::string)"[JACKDBUS] " + msg);
-#endif
-
-#if defined(LOG_TO_STD)
-	cerr << (std::string)"[JACKDBUS] " << msg << endl;
-#endif
+	_app->info_msg((std::string)"Jack: " + msg);
 }
 
